@@ -92,6 +92,14 @@ class RuleReference(Taggable, Expression):
     # Grammar name of referenced rule
     grammar_name: typing.Optional[str] = None
 
+    @property
+    def full_rule_name(self):
+        """Get fully qualified rule name."""
+        if self.grammar_name:
+            return f"{self.grammar_name}.{self.rule_name}"
+
+        return self.rule_name
+
 
 @dataclass
 class SlotReference(Substitutable, Taggable, Expression):
@@ -107,6 +115,7 @@ class ParseMetadata:
 
     file_name: str
     line_number: int
+    intent_name: typing.Optional[str] = None
 
 
 @dataclass
@@ -192,7 +201,7 @@ def walk_expression(
             assert isinstance(new_body, Sentence), f"Expected Sentence, got {new_body}"
             expression.rule_body = new_body
     elif isinstance(expression, RuleReference):
-        key = f"<{expression.rule_name}>"
+        key = f"<{expression.full_rule_name}>"
         if replacements and (key in replacements):
             key_replacements = replacements[key]
             for i in range(len(key_replacements)):
@@ -440,6 +449,10 @@ def parse_expression(
                     # Use entire name
                     rule.rule_name = rule_name
 
+                    if metadata:
+                        # Use intent name for grammar name
+                        rule.grammar_name = metadata.intent_name
+
                 rule.text = text[current_index:next_index]
                 last_group.items.append(rule)
                 last_taggable = rule
@@ -615,6 +628,10 @@ def parse_expression(
         last_group.items.extend(words)
 
     if last_group:
+        if not last_group.text:
+            # Fix text
+            last_group.text = " ".join(item.text for item in last_group.items)
+
         if len(last_group.items) == 1:
             # Simplify final group
             assert root is not None, parse_error(
@@ -624,9 +641,18 @@ def parse_expression(
                 metadata=metadata,
             )
             root.items[-1] = last_group.items[0]
-        elif not last_group.text:
-            # Fix text
-            last_group.text = " ".join(item.text for item in last_group.items)
+
+            # Force text to be fixed
+            root.text = ""
+
+    if root and (not root.text):
+        # Fix text
+        if root.type == SequenceType.ALTERNATIVE:
+            # Pipe separated
+            root.text = " | ".join(item.text for item in root.items)
+        else:
+            # Space separated
+            root.text = " ".join(item.text for item in root.items)
 
     if end and (not found):
         # Signal end not found
