@@ -32,7 +32,7 @@ def expression_to_graph(
     graph: nx.DiGraph,
     source_state: int,
     replacements: typing.Optional[ReplacementsType] = None,
-    empty_substitution: bool = False,
+    empty_substitution: int = 0,
     grammar_name: typing.Optional[str] = None,
     count_dict: typing.Optional[typing.Dict[Expression, int]] = None,
     rule_grammar: str = "",
@@ -44,7 +44,7 @@ def expression_to_graph(
     # Handle sequence substitution
     if isinstance(expression, Substitutable) and expression.substitution:
         # Ensure everything downstream outputs nothing
-        empty_substitution = True
+        empty_substitution += 1
 
     # Handle tag begin
     if isinstance(expression, Taggable) and expression.tag:
@@ -60,7 +60,7 @@ def expression_to_graph(
 
         if expression.tag.substitution:
             # Ensure everything downstream outputs nothing
-            empty_substitution = True
+            empty_substitution += 1
 
     # Handle converters begin
     begin_converters: typing.List[str] = []
@@ -131,7 +131,7 @@ def expression_to_graph(
         next_state = len(graph)
         graph.add_node(next_state, word=word.text)
 
-        if (word.substitution is None) and (not empty_substitution):
+        if (word.substitution is None) and (empty_substitution <= 0):
             # Single word input/output
             graph.add_edge(
                 source_state,
@@ -155,7 +155,7 @@ def expression_to_graph(
 
             # Add word output(s)
             olabels = [word.text] if (word.substitution is None) else word.substitution
-            if not empty_substitution:
+            if empty_substitution <= 0:
                 source_state = add_substitution(graph, olabels, source_state)
     elif isinstance(expression, RuleReference):
         # Reference to a local or remote rule
@@ -212,7 +212,9 @@ def expression_to_graph(
                 graph,
                 source_state,
                 replacements=replacements,
-                empty_substitution=(empty_substitution or bool(slot_ref.substitution)),
+                empty_substitution=(
+                    empty_substitution + (1 if slot_ref.substitution else 0)
+                ),
                 grammar_name=grammar_name,
                 count_dict=count_dict,
                 rule_grammar=rule_grammar,
@@ -231,7 +233,11 @@ def expression_to_graph(
     # Handle sequence substitution
     if isinstance(expression, Substitutable) and expression.substitution:
         # Output substituted word(s)
-        source_state = add_substitution(graph, expression.substitution, source_state)
+        empty_substitution -= 1
+        if empty_substitution <= 0:
+            source_state = add_substitution(
+                graph, expression.substitution, source_state
+            )
 
     # Handle converters end
     end_converters: typing.List[str] = []
@@ -241,16 +247,6 @@ def expression_to_graph(
     if isinstance(expression, Taggable) and expression.tag:
         end_converters.extend(expression.tag.converters)
 
-    # Create end transitions for each converter
-    for converter_name in end_converters:
-        next_state = len(graph)
-        olabel = f"__converted__{converter_name}"
-        label = f"!{olabel}"
-        graph.add_edge(
-            source_state, next_state, ilabel="", olabel=maybe_pack(olabel), label=label
-        )
-        source_state = next_state
-
     # Handle tag end
     if isinstance(expression, Taggable) and expression.tag:
         # Handle tag substitution
@@ -260,6 +256,20 @@ def expression_to_graph(
                 graph, expression.tag.substitution, source_state
             )
 
+        # Create end transitions for each converter
+        for converter_name in end_converters:
+            next_state = len(graph)
+            olabel = f"__converted__{converter_name}"
+            label = f"!{olabel}"
+            graph.add_edge(
+                source_state,
+                next_state,
+                ilabel="",
+                olabel=maybe_pack(olabel),
+                label=label,
+            )
+            source_state = next_state
+
         # End tag
         next_state = len(graph)
         tag = expression.tag.tag_text
@@ -269,6 +279,20 @@ def expression_to_graph(
             source_state, next_state, ilabel="", olabel=maybe_pack(olabel), label=label
         )
         source_state = next_state
+    else:
+        # Create end transitions for each converter
+        for converter_name in end_converters:
+            next_state = len(graph)
+            olabel = f"__converted__{converter_name}"
+            label = f"!{olabel}"
+            graph.add_edge(
+                source_state,
+                next_state,
+                ilabel="",
+                olabel=maybe_pack(olabel),
+                label=label,
+            )
+            source_state = next_state
 
     return source_state
 
@@ -376,7 +400,7 @@ def sentences_to_graph(
         olabel = f"__label__{intent_name}"
         label = f":{olabel}"
 
-        edge_kwargs = {}
+        edge_kwargs: typing.Dict[str, typing.Any] = {}
         if add_intent_weights and (num_intents > 1):
             edge_kwargs["sentence_count"] = intent_counts.get(intent_name, 1)
             edge_kwargs["weight"] = intent_weights.get(intent_name, 0)
